/*
 * Author: Brendan Le Foll <brendan.le.foll@intel.com>
 * Author: Thomas Ingleby <thomas.c.ingleby@intel.com>
 * Author: Matthias Hahn <matthias.hahn@intel.com>
 * Copyright (c) 2014 Intel Corporation.
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#define LIGHTRESH 2300
#define SCROLLSPEED 300000

#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include "mraa.hpp"

// for buzzer
#include <signal.h>
#include "buzzer.h"

// for Light sensor
#include "grove.h"

// for LCD
#include "jhd1313m1.h"


// Subroutines
void Stimer(void); // Study Timer
void Rtimer(void); // Resting Timer

// Global vars

// TIME
int hh = 0; // 4-5 col in LCD
int mm = 0; // 7-8 col in LCD
int ss = 0; // 10-11 col in LCD
char buff[100];
std::string timeStr;

// TIME2
int rhh = 0; // 4-5 col in LCD
int rmm = 0; // 7-8 col in LCD
int rss = 0; // 10-11 col in LCD
char rbuff[100];
std::string rtimeStr;

// LCD  - 2 ROWS / 16 COLS
// 0x62 RGB_ADDRESS, 0x3E LCD_ADDRESS
upm::Jhd1313m1 *lcd = new upm::Jhd1313m1(6, 0x3E, 0x62);


int main (int argc, char **argv)
{
	lcd->setColor(50, 255, 50); // sets LCD to GREEN

	// Create the light sensor using AIO pin 0
	upm::GroveLight* light = new upm::GroveLight(0);

	// Create led using DIO pin 2
	upm::GroveLed* led = new upm::GroveLed(2);

	// Read the input and print both the raw value and a rough lux value,
	// waiting one second between readings
	int lightIntensity;

    // BOTTON CODE
    mraa_gpio_context gpio;
    gpio = mraa_gpio_init(6);
    mraa_gpio_dir(gpio, MRAA_GPIO_IN);

    bool buttonPressed = 0; //starts with the button unpressed
    bool lamp = false; 		// starts with the lamp OFF

    // LCD WELCOMING
	lcd->setCursor(0, 0);
	lcd->write("WELCOME");

	for (int i = 0; i < 16; i++) // scrolls until WELCOME dissapear
	{
		lcd->scroll(false); //scrolls to the right
		usleep(SCROLLSPEED); // waits
	}

	// Initialize LCD
	lcd->clear();
	sleep(1);
	lcd->setCursor(0, 0);
	lcd->write(" STUDY 00:00:00 ");
	lcd->setCursor(1, 0);
	lcd->write(" BREAK 00:00:00 ");


    for (;;)
    {
    	// Checks button state
    	buttonPressed = mraa_gpio_read(gpio);

    	// Checks light intensity
    	lightIntensity = int(light->raw_value());
    	fprintf(stdout, "Light %d\n", lightIntensity);


    	if(buttonPressed) // if button pressed
    	{
    		fprintf(stdout, "SITTING DOWN\n");
    		lcd->setColor(50, 50, 255); // sets LCD to BLUE
    		Stimer(); // Increments the Study timer

    		if(lightIntensity < LIGHTRESH)
    		{
    			lamp = true;
    		}
    		else
    		{
    			lamp = false;
    		}
    	}
    	else // if button unpressed
		{
    		fprintf(stdout, "STANDING ON\n");
    		lcd->setColor(255, 10, 10); // sets LCD to RED
    		Rtimer(); // Increments resting timer
    		// Turns OFF the lamp
    		lamp = false;
		}

    	if(lamp)
    	{
    		fprintf(stdout, "LAMP ON\n");
    		led->on();
    	}
    	else
    	{
    		fprintf(stdout, "LAMP OFF\n");
    		led->off();
    	}

    	sleep(1);	// Waits for 1 second
    }


    mraa_gpio_close(gpio);
	lcd->close();			// finish using the LCD
}

void Stimer() // Study timer
{
	//lcd->setCursor(0, 4);
	//lcd->write("00:00:00");

	ss++;

	if (ss > 59)
	{
		ss = 00;
		mm++;
	}

	if (mm > 59)
	{
		mm = 00;
		hh++;
	}

	if (hh > 24)
	{
		hh = 00;
		mm = 00;
		ss = 00;
	}

	lcd->setCursor(0, 0);
	sprintf(buff, " STUDY %02i:%02i:%02i ", hh, mm, ss);
	timeStr = buff;
	lcd->write(timeStr);
}

void Rtimer() // Resting timer
{
	//lcd->setCursor(0, 4);
	//lcd->write("00:00:00");

	rss++;

	if (rss > 59)
	{
		rss = 00;
		rmm++;
	}

	if (rmm > 59)
	{
		rmm = 00;
		rhh++;
	}

	if (rhh > 24)
	{
		rhh = 00;
		rmm = 00;
		rss = 00;
	}

	lcd->setCursor(1, 0);
	sprintf(rbuff, " BREAK %02i:%02i:%02i ", rhh, rmm, rss);
	rtimeStr = rbuff;
	lcd->write(rtimeStr);
}
